/*
Layout Manager steuern, wie Komponenten (Knpfe, Listen, ...) innerhalb eines Containers angeordnet werden.
Das hilft, sich auf verschiedene Fenstergren etc. einzustellen. Auch bzgl. Plattformunabhngigkeit
und austauschbarem Look&Feel ergeben sich Vorteile, wenn man die Positionen/Gren von Komponenten nicht
als soundsoviel Pixel fest einprogrammiert.
*/

// Tipp: Anwendungsfenster auf verschiedene Formate ziehen

package layoutmanager;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class LayoutManagerDemo
{
  public static void main(String[] args)
  {
    JFrame myFrame = new JFrame("LayoutManagerDemo");
    myFrame.addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){System.exit(0);}});

    myFrame.setSize(600, 600);
    myFrame.setLocation(100, 100);

    JTabbedPane myTabbedPane = new JTabbedPane();

    JPanel flowLayoutPanel    = new JPanel();
    buildFlowLayoutPanel   (flowLayoutPanel);

    JPanel gridLayoutPanel    = new JPanel();
    buildGridLayoutPanel   (gridLayoutPanel);

    JPanel borderLayoutPanel  = new JPanel();
    buildBorderLayoutPanel (borderLayoutPanel);

    JPanel boxLayoutPanel     = new JPanel();
    buildBoxLayoutPanel    (boxLayoutPanel);

    JPanel gridBagLayoutPanel = new JPanel();
    buildGridBagLayoutPanel(gridBagLayoutPanel);

    JPanel absolutePanel      = new JPanel();
    buildAbsolutePanel     (absolutePanel);

    JPanel compositePanel       = new JPanel();
    buildCompositePanel    (compositePanel);

    myTabbedPane.addTab("Flow Layout",     flowLayoutPanel);
    myTabbedPane.addTab("Grid Layout",     gridLayoutPanel);
    myTabbedPane.addTab("Border Layout",   borderLayoutPanel);
    myTabbedPane.addTab("Box Layout",      boxLayoutPanel);
    myTabbedPane.addTab("Grid Bag Layout", gridBagLayoutPanel);
    myTabbedPane.addTab("absolut",         absolutePanel);
    myTabbedPane.addTab("kombiniert",      compositePanel);

    myFrame.getContentPane().add(myTabbedPane);

    myFrame.setVisible(true);
  }

  static void buildFlowLayoutPanel(JPanel flowLayoutPanel)
  {
    flowLayoutPanel.setLayout(new FlowLayout()); // eigentlich berflssig, weil Standard fr diesen Containertyp
    // Flow Layout versucht, Zeile um Zeile von links nach rechts zu fllen.

    JButton a = new JButton("A");
    flowLayoutPanel.add(a);

    JButton b = new JButton("B");
    flowLayoutPanel.add(b);

    JButton c = new JButton("C");
    flowLayoutPanel.add(c);

    JButton d = new JButton("D");
    flowLayoutPanel.add(d);

    BigButton e = new BigButton("E");
    flowLayoutPanel.add(e);

    JButton f = new JButton("F");
    flowLayoutPanel.add(f);

    final JLabel l = new JLabel("zuletzt geklickt: ---");
    // final (vgl. const in C++) ntig, um aus innerer Klasse (s.u.) auf lokale Variable zuzugreifen
    l.setPreferredSize(new Dimension(200, 20));
    flowLayoutPanel.add(l);

    // Events verdrahten
    a.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: A");}});
    b.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: B");}});
    c.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: C");}});
    d.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: D");}});
    e.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: E");}});
    f.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: F");}});
  }

  static void buildGridLayoutPanel(JPanel gridLayoutPanel)
  {
    gridLayoutPanel.setLayout(new GridLayout(0, 2));
    // Grid Layout versucht, gleichmige Spalten (hier 2) und Zeilen (hier beliebig) zu bilden.

    JButton a = new JButton("A");
    gridLayoutPanel.add(a);

    JButton b = new JButton("B");
    gridLayoutPanel.add(b);

    JButton c = new JButton("C");
    gridLayoutPanel.add(c);

    JButton d = new JButton("D");
    gridLayoutPanel.add(d);

    BigButton e = new BigButton("E"); // Der wird hier nicht grer!
    gridLayoutPanel.add(e);

    JButton f = new JButton("F");
    gridLayoutPanel.add(f);

    final JLabel l = new JLabel("zuletzt geklickt: ---");
    // final (vgl. const in C++) ntig, um aus innerer Klasse (s.u.) auf lokale Variable zuzugreifen
    gridLayoutPanel.add(l);

    // Events verdrahten
    a.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: A");}});
    b.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: B");}});
    c.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: C");}});
    d.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: D");}});
    e.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: E");}});
    f.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: F");}});
  }

  static void buildBorderLayoutPanel(JPanel borderLayoutPanel)
  {
    borderLayoutPanel.setLayout(new BorderLayout());
    // Border Layout verteilt Elemente an den Rndern und setzt eines in die Mitte.

    JButton a = new JButton("A");
    borderLayoutPanel.add(a, BorderLayout.NORTH);

    JButton b = new JButton("B");
    borderLayoutPanel.add(b, BorderLayout.EAST);

    JButton c = new JButton("C");
    borderLayoutPanel.add(c, BorderLayout.SOUTH);

    BigButton d = new BigButton("D");
    borderLayoutPanel.add(d, BorderLayout.WEST);

    final JLabel l = new JLabel("zuletzt geklickt: ---");
    // final (vgl. const in C++) ntig, um aus innerer Klasse (s.u.) auf lokale Variable zuzugreifen
    borderLayoutPanel.add(l, BorderLayout.CENTER);

    // Events verdrahten
    a.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: A");}});
    b.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: B");}});
    c.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: C");}});
    d.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: D");}});
  }

  static void buildBoxLayoutPanel(JPanel boxLayoutPanel)
  {
    // Bei Box Layout einfacher den vorgefertigten Container einbinden statt
    // den Box-Layout-Manager per Hand zu fttern:
    Container myBox = Box.createHorizontalBox(); // entsprechend auch createVerticalBox
    boxLayoutPanel.setLayout(new BorderLayout());
    boxLayoutPanel.add(myBox, BorderLayout.CENTER); // Damit sich myBox voll ausdehnt.
    // Box Layout versucht, die Elemente gleichmig mit flexiblen Zwischenrumen (glue = Kleber)
    // in einer Reihe anzuordnen.

    JButton a = new JButton("A");
    myBox.add(a);

    JButton b = new JButton("B");
    myBox.add(b);

    myBox.add(Box.createHorizontalGlue()); // variabler Abstand

    JButton c = new JButton("C");
    myBox.add(c);

    JButton d = new JButton("D");
    myBox.add(d);

    myBox.add(Box.createHorizontalStrut(20)); // fester Abstand

    BigButton e = new BigButton("E");
    myBox.add(e);

    JButton f = new JButton("F");
    myBox.add(f);

    final JLabel l = new JLabel("zuletzt geklickt: ---");
    // final (vgl. const in C++) ntig, um aus innerer Klasse (s.u.) auf lokale Variable zuzugreifen
    l.setPreferredSize(new Dimension(200, 20));
    myBox.add(l);

    // Events verdrahten
    a.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: A");}});
    b.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: B");}});
    c.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: C");}});
    d.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: D");}});
    e.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: E");}});
    f.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: F");}});
  }

  static void buildGridBagLayoutPanel(JPanel gridBagLayoutPanel)
  {
    gridBagLayoutPanel.setLayout(new GridBagLayout());
    // Grid Layout versucht, Spalten und Zeilen zu bilden.
    // Dabei knnen Elemente ber mehrere Spalten und Zeilen reichen.
    // Die Breite bzw. Hhe von Spalten und Zeilen richtet sich nach Gewichten und Fllbefehlen.

    GridBagConstraints cs = new GridBagConstraints();

    JButton a = new JButton("A");
    cs.gridx = 0;
    cs.gridy = 0;
    cs.weightx = 1.0;
    cs.weighty = 1.0;
    cs.fill = GridBagConstraints.BOTH;
    cs.gridheight = 1;
    cs.gridwidth = 1;
    gridBagLayoutPanel.add(a, cs);

    JButton b = new JButton("B");
    cs.gridx = 2;
    cs.gridy = 0;
    cs.weightx = 2.0;
    cs.weighty = 1.0;
    cs.fill = GridBagConstraints.BOTH;
    cs.gridheight = 1;
    cs.gridwidth = 1;
    gridBagLayoutPanel.add(b, cs);

    JButton c = new JButton("C");
    cs.gridx = 0;
    cs.gridy = 1;
    cs.weightx = 1.0;
    cs.weighty = 0.5;
    cs.fill = GridBagConstraints.BOTH;
    cs.gridheight = 1;
    cs.gridwidth = 1;
    gridBagLayoutPanel.add(c, cs);

    JButton d = new JButton("D");
    cs.gridx = 1;
    cs.gridy = 1;
    cs.weightx = 1.0;
    cs.weighty = 0.5;
    cs.fill = GridBagConstraints.BOTH;
    cs.gridheight = 1;
    cs.gridwidth = 2;
    gridBagLayoutPanel.add(d, cs);

    BigButton e = new BigButton("E");
    cs.gridx = 0;
    cs.gridy = 2;
    cs.weightx = 1.0;
    cs.weighty = 1.0;
    cs.fill = GridBagConstraints.BOTH;
    cs.gridheight = 1;
    cs.gridwidth = 1;
    gridBagLayoutPanel.add(e, cs);

    JButton f = new JButton("F");
    cs.gridx = 1;
    cs.gridy = 2;
    cs.weightx = 1.0;
    cs.weighty = 1.0;
    cs.fill = GridBagConstraints.HORIZONTAL;
    cs.gridheight = 1;
    cs.gridwidth = 1;
    gridBagLayoutPanel.add(f, cs);

    final JLabel l = new JLabel("zuletzt geklickt: ---");
    // final (vgl. const in C++) ntig, um aus innerer Klasse (s.u.) auf lokale Variable zuzugreifen
    cs.gridx = 0;
    cs.gridy = 3;
    cs.weightx = 1.0;
    cs.weighty = 1.0;
    cs.fill = GridBagConstraints.BOTH;
    cs.gridheight = 1;
    cs.gridwidth = 1;
    gridBagLayoutPanel.add(l, cs);

    // Events verdrahten
    a.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: A");}});
    b.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: B");}});
    c.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: C");}});
    d.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: D");}});
    e.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: E");}});
    f.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: F");}});
  }

  static void buildAbsolutePanel(JPanel absolutePanel)
  {
    absolutePanel.setLayout(null);
    // kein Layout-Manager

    JButton a = new JButton("A");
    absolutePanel.add(a);
    a.setSize(a.getPreferredSize());
    a.setLocation(100, 100);

    JButton b = new JButton("B");
    absolutePanel.add(b);
    b.setSize(b.getPreferredSize());
    b.setLocation(50, 300);

    JButton c = new JButton("C");
    absolutePanel.add(c);
    c.setSize(c.getPreferredSize());
    c.setLocation(200, 150);

    final JButton d = new JButton("D (Zieh mich!)");
    // final (vgl. const in C++) ntig, um aus innerer Klasse (s.u.) auf lokale Variable zuzugreifen
    absolutePanel.add(d);
    d.setSize(d.getPreferredSize());
    d.setLocation(200, 50);

    BigButton e = new BigButton("E");
    absolutePanel.add(e);
    e.setSize(e.getPreferredSize());
    e.setLocation(250, 300);

    JButton f = new JButton("F");
    absolutePanel.add(f);
    f.setSize(f.getPreferredSize());
    f.setLocation(150, 200);

    final JLabel l = new JLabel("zuletzt geklickt: ---");
    // final (vgl. const in C++) ntig, um aus innerer Klasse (s.u.) auf lokale Variable zuzugreifen
    absolutePanel.add(l);
    l.setSize(l.getPreferredSize());
    l.setLocation(300, 200);

    // Events verdrahten
    a.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: A");}});
    b.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: B");}});
    c.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: C");}});
    d.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: D");}});
    e.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: E");}});
    f.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent e){l.setText("zuletzt geklickt: F");}});

    // kleine Spielerei, die mit einem Layout-Manager schwierig wrde
    d.addMouseMotionListener(
      new MouseMotionAdapter()
      {
        int startX = 0;
        int startY = 0;

        public void mouseMoved(MouseEvent e)
        {
          startX = e.getX();
          startY = e.getY();
        }

        public void mouseDragged(MouseEvent e)
        {
          d.setLocation(d.getX()+e.getX()-startX, d.getY()+e.getY()-startY);
        }
      }
    ); // Ende addMouseMotionListener

  }

  static void buildCompositePanel(JPanel compositePanel)
  {
    compositePanel.setLayout(new BorderLayout());

    JPanel subNorthLayoutPanel = new JPanel();
    compositePanel.add(subNorthLayoutPanel, BorderLayout.NORTH);
    buildBoxLayoutPanel(subNorthLayoutPanel);

    JPanel subCenterPanel = new JPanel();
    compositePanel.add(subCenterPanel, BorderLayout.CENTER);
    buildAbsolutePanel(subCenterPanel);

    JPanel subWestPanel = new JPanel();
    compositePanel.add(subWestPanel, BorderLayout.WEST);
    buildGridLayoutPanel(subWestPanel);
  }
}

class BigButton
extends JButton
{
  BigButton(String name)
  {
    super(name);
  }

  public Dimension getMinimumSize()
  {
    int newWidth  = 2 * super.getMinimumSize().width;
    int newHeight = 2 * super.getMinimumSize().height;
    return new Dimension(newWidth, newHeight);
  }

  public Dimension getMaximumSize()
  {
    int newWidth  = 2 * super.getMaximumSize().width;
    int newHeight = 2 * super.getMaximumSize().height;
    return new Dimension(newWidth, newHeight);
  }

  public Dimension getPreferredSize()
  {
    int newWidth  = 2 * super.getPreferredSize().width;
    int newHeight = 2 * super.getPreferredSize().height;
    return new Dimension(newWidth, newHeight);
  }
}